@@ -1,8 +1,9 @@
+# -*- mode: cperl; tab-width: 8; indent-tabs-mode: nil; basic-offset: 2 -*-
+# vim:ts=8:sw=2:et:sta:sts=2
 #########
 # Author:        rmp
-# Last Modified: $Date: 2010-11-07 07:59:39 +0000 (Sun, 07 Nov 2010) $
-# Id:            $Id: Expr.pm 53 2010-11-07 07:59:39Z bolav $
-# Source:        $Source$
+# Last Modified: $Date: 2012-09-12 09:42:30 +0100 (Wed, 12 Sep 2012) $
+# Id:            $Id: Expr.pm 71 2012-09-12 08:42:30Z zerojinx $
 # $HeadURL: https://text-sass.svn.sourceforge.net/svnroot/text-sass/trunk/lib/Text/Sass/Expr.pm $
 #
 package Text::Sass::Expr;
@@ -11,7 +12,7 @@ use warnings;
 use Carp;
 use Readonly;
 
-our $VERSION = 0.1;
+our $VERSION = q[1.0.0];
 
 Readonly::Scalar our $SHADE_MAX => 255;
 
@@ -76,6 +77,14 @@ sub expr {
   my ($p2, $u2) = @{$pkg->units($part2)};
   return if(!defined $p1);
 
+  if(!$u1) {
+    $u1 = q[];
+  }
+
+  if(!$u2) {
+    $u2 = q[];
+  }
+
   if(!$u1 && $u2) {
     $u1 = $u2;
   }
@@ -118,7 +127,7 @@ sub units {
     return [$pkg->hex_to_rgb($token), q[#]];
   }
 
-  my ($val, $units) = $token =~ /([\d\.]+)(px|pt|pc|em|ex|mm|cm|in|%|)/smx;
+  my ($val, $units) = $token =~ /([\d.]+)(px|pt|pc|em|ex|mm|cm|in|%|)/smx;
 
   return [$val, $units];
 }
@@ -149,13 +158,15 @@ sub convert {
 1;
 __END__
 
+=encoding utf8
+
 =head1 NAME
 
 Text::Sass::Expr
 
 =head1 VERSION
 
-$LastChangedRevision: 53 $
+$LastChangedRevision: 71 $
 
 =head1 SYNOPSIS
 
